## Supplementary files for an article entitled 
## "How big is that manta ray? A novel and non-invasive method for measuring reef manta rays using small drones" 
## submitted to drones MDPI special issue "Drones for Biodiversity and Conservation" 
## (https://www.mdpi.com/journal/drones/special_issues/drones_biodiversity_conservation)

# The code is written by Edy Setyawan (edysetyawan@gmail.com) and Ben C. Stevenson (ben.stevenson@auckland.ac.nz)

# Questions? Please contact us!

# 1. Load Packages & Functions -------------------------------------------

source("functions.R") # main functions used for analysing drone measurement data
library(readxl)
library(mvtnorm)
library(TMB)
library(viridis)

# 2. Data Import and Preparation ------------------------------------------

## importing data "drone.data.xlsx" and prepare 2 datasets: 
## drone.df (measurement data) and individual data (single population and grouped based on maturity)
drone.df <- read_excel("drone.data.xlsx", sheet = "measurements") # drone measurement data

individual.single.df <- read_excel("drone.data.xlsx", sheet = "individuals") # a single group in population
individual.single.df$group <- as.numeric(individual.single.df$group)
summary(individual.single.df) # summarise data

individual.group.df <- read_excel("drone.data.xlsx", sheet = "individuals") 
# three groups in population (Group1 = mature females, Group2 = mature males, and Group3 = unsexed individuals)
individual.group.df$group <- ifelse(individual.group.df$sex == "Male", 1, 
                                    ifelse(individual.group.df$sex == "Female", 2, 3))
individual.group.df$group <- as.numeric(individual.group.df$group)
summary(individual.group.df) # summarise data

# 3. Model fitting --------------------------------------------------------

## Let's compile the model. 
## Note = This will take a while and this process will create two files in your working directory (model.o and model.so)
compile("model.cpp") # Compilation only needs to be done once!

## 3A. Fitting model: a single group in population
fit.single <- fit.hmv(drone.df, individual.single.df[,-5]) # -5 = excluding column 5th (manta color morph) from model fitting. We don't need this in model fitting for now
save(fit.single, file = "drone.fit.single.RData") # save model fit as .RData for later use

## 3B. Fitting model: 3 groups in population based on maturity
fit.group <- fit.hmv(drone.df = drone.df, individual.df = individual.group.df[,-5]) # -5 = excluding column 5th (manta color morph) from model fitting. We don't need this in model fitting for now

save(fit.group, file = "drone.fit.group.RData") # save model fit as .RData for later use

## Alternatively, model fits can be loaded below from the saved .RData
load("drone.fit.single.RData")
load("drone.fit.group.RData")

# 4. Model summary and exploring model fit -----------------------------------

## Using model fit for drone data measured from manta rays as a single population
## Note = Dim1 (DW), Dim2 (DL), Dim3 (CW)
summary(fit.single, par = "pop.mu") # the population mean of each dimension for a group 
summary(fit.single, par = "pop.sd") # the standard deviations of each dimension for a group
summary(fit.single, par = "pop.rho") # correlations between dimensions for a group
summary(fit.single, par = "me.sd") # the standard deviations of measurement errors of each dimension

## looking at the estimated true measurement for three dimensions (in columns) of each individual (in rows) 
fit.single$u.ests # estimated true measurement of the three dimensions (DW, DL, CW) for each individual
fit.single$u.ests[,1] # estimated true measurement of DW for each individual
fit.single$u.ests[,2] # estimated true measurement of DL for each individual
fit.single$u.ests[,3] # estimated true measurement of CW for each individual
fit.single$u.ses # the standard error of the true measurement of three dimensions (DW, DL, CW) for each individual (in rows)

## Plotting model fit
## plotting model fit based on a single group in population
## Note = dims = c(x-axis, y-axis). 1 = Disc width, 2 = Disc length, 3 = Cranial width
plot(fit.single, group = 1, dims = c(2, 1)) # for single population (only 1 group) looking at DL vs DW
plot(fit.single, group = 1, dims = c(3, 2)) # for single population (only 1 group) looking at CW vs DL

## plotting model fit based on 3 groups in population
plot(object = fit.group, group = 1, dims = c(2, 1)) 
# for three different groups (1 = mature females, 2 = mature males, 3 = unsexed individuals)

# 5. Testing prediction using the model --------------------------------------------------------

## Testing prediction using model using single population from
## 'incomplete_measurement.csv' Let's predict DW for a manta ray in
## with DL of 100 cm and CW of 60 cm. Only works if you have fitted
## the model rather than loading the .RData objects.

## Below we use group = 1 as we use model fit for a single population
## Dimension(s) we want to predict should be in NA
new.individual.df <- data.frame(id = 1, group = 1, x1 = NA, x2 = 100, x3 = 60) # x1 = DW; x2 = DL; x3 = CW

predict(fit.single, new.individual.df = new.individual.df) 
## Result = The estimated true value of DW is 232.5 cm with a standard error of 6.24 cm.

## What if we predict using only a single measurement (CW = 60 cm)?
new.individual.df <- data.frame(id = 1, group = 1, x1 = NA, x2 = NA, x3 = 60) # x1 = DW; x2 = DL; x3 = CW
predict(fit.single, new.individual.df = new.individual.df) 
## Result = The estimated true value of DW is 236.5 cm with a standard error of 7.63 cm; 
## while the estimate of DL is 103.8 cm with standard error of 4.22 cm. 
## See the difference in predicting using one known dimension and two known dimensions.