#include <TMB.hpp>
#include <fenv.h>

// Function to construct a covariance matrix given correlations and
// standard deviations.
template<class Type>
matrix<Type> construct_cov (const vector<Type> &rho, const vector<Type> &sd){
  int n_dims = sd.size();
  matrix<Type> cor_mat(n_dims, n_dims);
  matrix<Type> vcov_mat(n_dims, n_dims);
  int k = 0;
  for (int i = 0; i < n_dims - 1; i++){
    for (int j = i + 1; j < n_dims; j++){
      cor_mat(i, j) = rho(k);
      cor_mat(j, i) = rho(k);
      k++;
    }
  }
  for (int i = 0; i < n_dims; i++){
    for (int j = 0; j < n_dims; j++){
      if (i == j){
	cor_mat(i, j) = 1;
      }
      vcov_mat(i, j) = cor_mat(i, j)*sd(i)*sd(j);
    }
  }
  return vcov_mat;
}
// Overloading function for matrix input
template<class Type>
array<Type> construct_cov (const matrix<Type> &rho, const matrix<Type> &sd){
  int n_groups = sd.rows();
  int n_dims = sd.cols();
  array<Type> vcov_arr(n_dims, n_dims, n_groups);
  vector<Type> rho_vec(n_dims);
  vector<Type> sd_vec(n_dims);
  matrix<Type> vcov_mat(n_dims, n_dims);
  for (int g = 0; g < n_groups; g++){
    rho_vec = rho.row(g);
    sd_vec = sd.row(g);
    vcov_mat = construct_cov(rho_vec, sd_vec);
    for (int i = 0; i < n_dims; i++){
      for (int j = 0; j < n_dims; j++){
	vcov_arr(i, j, g) = vcov_mat(i, j);
      }
    }
  }
  return vcov_arr;
}

// Extract submatrix from array.
template<class Type>
matrix<Type> extract_sub (array<Type> &arr, const int &index){
  int nr = arr.dim(0);
  int nc = arr.dim(1);
  matrix<Type> out(nr, nc);
  for (int i = 0; i < nr; i++){
    for (int j = 0; j < nc; j++){
      out(i, j) = arr(i, j, index);
    }
  }
  return(out);
}

// Function for a multivariate lognormal distribution.
template<class Type>
Type dmvlnorm (const vector<Type> &x, const vector<Type> &meanlog, const matrix<Type> &sigmalog, int give_log = 0){
  int n = x.size();
  vector<Type> x_log = log(x);
  vector<Type> x_log_centred(n);
  for (int i = 0; i < n; i++){
    x_log_centred(i) = x_log(i) - meanlog(i);
  }
  Type out = -density::MVNORM(sigmalog)(x_log_centred) - sum(log(x));
  if (give_log == 0){
    out = exp(out);
  }
  return out;
}

template<class Type>
Type objective_function<Type>::operator() ()
{
  // Number of mantas.
  DATA_INTEGER(n_mantas);
  // Number of dimensions.
  DATA_INTEGER(n_dims);
  // Number of groups.
  DATA_INTEGER(n_groups);
  // Observed photo measurements.
  DATA_MATRIX(y);
  // Animal ID for each photo.
  DATA_IVECTOR(y_id);
  // Animal group for each individual.
  DATA_IVECTOR(group);
  // Indicators for whether or not u is known.
  DATA_IMATRIX(u_is_known);
  // Known u values.
  DATA_VECTOR(u_known);
  // Indicator for distribution of measurement error.
  DATA_INTEGER(error_mvn);
  // Number of values to predict for each dimension.
  DATA_INTEGER(n_pred_vals);
  // Values of each dimension to predict for the others.
  DATA_MATRIX(x_pred_vals);
  // Indicator of for no drone data.
  DATA_INTEGER(no_y);
  // Indicator for doing predictions based on a range of x-values.
  DATA_INTEGER(do_preds);
  // True manta ray dimensions.
  PARAMETER_VECTOR(u_unknown);
  // Putting together the u matrix of both known and unknown u.
  matrix<Type> u(n_mantas, n_dims);
  int k_known = 0;
  int k_unknown = 0;
  for (int i = 0; i < n_mantas; i++){
    for (int j = 0; j < n_dims; j++){
      if (u_is_known(i, j) == 1){
	u(i, j) = u_known(k_known);
	k_known++;
      } else {
	u(i, j) = u_unknown(k_unknown);
	k_unknown++;
      }
    }
  }
  // Correlations between true dimensions.
  PARAMETER_MATRIX(logit_pop_rho);
  matrix<Type> pop_rho(logit_pop_rho.rows(), logit_pop_rho.cols());
  for (int i = 0; i < logit_pop_rho.rows(); i++){
    for (int j = 0; j < logit_pop_rho.cols(); j++){
      pop_rho(i, j) = exp(logit_pop_rho(i, j))/(1 + exp(logit_pop_rho(i, j)));
    }
  }
  ADREPORT(pop_rho);
  // Standard deviation of the true dimensions between individuals within the population.
  PARAMETER_MATRIX(log_pop_sd);
  matrix<Type> pop_sd(log_pop_sd.rows(), log_pop_sd.cols());
  for (int i = 0; i < log_pop_sd.rows(); i++){
    for (int j = 0; j < log_pop_sd.cols(); j++){
      pop_sd(i, j) = exp(log_pop_sd(i, j));
    }
  }
  ADREPORT(pop_sd);  
  // Mean of each dimension in the population.
  PARAMETER_MATRIX(identity_pop_mu);
  matrix<Type> pop_mu = identity_pop_mu;
  ADREPORT(pop_mu);
  // Correlations in measurement error between dimensions.
  PARAMETER_VECTOR(logit_me_rho);
  vector<Type> me_rho = exp(logit_me_rho)/(1 + exp(logit_me_rho));
  ADREPORT(me_rho);
  // Standard deviation of measurement error for each dimension between photos of the same individual.
  PARAMETER_VECTOR(log_me_sd);
  vector<Type> me_sd = exp(log_me_sd);
  ADREPORT(me_sd);
  // Number of photos.
  int n_photos = y.rows();
  // Variance-covariance matrix for measurement error.
  matrix<Type> xi = construct_cov(me_rho, me_sd);
  // Initialising joint log-PDF.
  Type f = 0;
  // Joint PDF of y given u. Only used if y is provided.
  if (no_y == 0){
    for (int i = 0; i < n_photos; i++){
      if (error_mvn == 1){
	// Vector of centred photo measurement data.
	vector<Type> y_centred(n_dims);
	for (int j = 0; j < n_dims; j++){
	  y_centred(j) = y(i, j) - u(y_id(i) - 1, j);
	}
	f += density::MVNORM(xi)(y_centred);
      } else{
	vector<Type> log_u(n_dims);
	vector<Type> y_use(n_dims);
	for (int j = 0; j < n_dims; j++){
	  log_u(j) = log(u(y_id(i) - 1, j));
	  y_use(j) = y(i, j);
	}
	f -= dmvlnorm(y_use, log_u, xi, 1);
      }
    }
  }
  // Vector of centred true measurement data.
  vector<Type> u_centred(n_dims);
  // Variance-covariance matrix for the population.
  array<Type> sigma = construct_cov(pop_rho, pop_sd);
  for (int i = 0; i < n_mantas; i++){
    for (int j = 0; j < n_dims; j++){
      u_centred(j) = u(i, j) - pop_mu(group(i) - 1, j);
    }
    matrix<Type> sub_sigma = extract_sub(sigma, group(i) - 1);
    f += density::MVNORM(sub_sigma)(u_centred);
  }
  if (do_preds){
    // Calculating predicted y values from each x value.
    array<Type> y_pred_vals(n_dims*(n_dims - 1), n_pred_vals, n_groups);
    for (int g = 0; g < n_groups; g++){
      int ii = 0;
      int iii = 0;
      for (int i = 0; i < n_dims; i++){
	for (int j = i + 1; j < n_dims; j++){
	  for (int k = 0; k < n_pred_vals; k++){
	    // Predicting ith dimension based on value of jth dimension.
	    y_pred_vals(ii, k, g) = pop_mu(g, i) + pop_sd(g, i)/pop_sd(g, j)*pop_rho(g, iii)*(x_pred_vals(j, k) - pop_mu(g, j));
	    // Predicting jth dimension based on value of ith dimension.
	    y_pred_vals(ii + 1, k, g) = pop_mu(g, j) + pop_sd(g, j)/pop_sd(g, i)*pop_rho(g, iii)*(x_pred_vals(i, k) - pop_mu(g, i));
	  }
	  // Incrementing ii twice.
	  ii++; ii++;
	  // Incrementing iii once.
	  iii++;
	}
      }
    }
    ADREPORT(y_pred_vals);
  }
  // Reporting true measurements.
  ADREPORT(u);
  return f;
}
